@@ -1,11 +1,10 @@
 /* apt-dater - terminal-based remote package update manager
  *
  * Authors:
- *   Andre Ellguth <ellguth@ibh.de>
  *   Thomas Liske <liske@ibh.de>
  *
  * Copyright Holder:
- *   2008-2012 (C) IBH IT-Service GmbH [http://www.ibh.de/apt-dater/]
+ *   2008-2015 (C) IBH IT-Service GmbH [https://www.ibh.de/apt-dater/]
  *
  * License:
  *   This program is free software; you can redistribute it and/or modify
@@ -46,8 +45,12 @@
 #include "report.h"
 #endif
 
-#define VERSTEXT PACKAGE_STRING " - " __DATE__ " " __TIME__ "\n\n" \
-  "Copyright Holder: IBH IT-Service GmbH [http://www.ibh.net/]\n\n" \
+#ifndef SOURCE_DATE_UTC
+#error SOURCE_DATE_UTC is undefined!
+#endif
+
+#define VERSTEXT PACKAGE_STRING " - " SOURCE_DATE_UTC "\n\n" \
+  "Copyright Holder: IBH IT-Service GmbH [https://www.ibh.net/]\n\n" \
   "This program is free software; you can redistribute it and/or modify\n" \
   "it under the terms of the GNU General Public License as published by\n" \
   "the Free Software Foundation; either version 2 of the License, or\n" \
@@ -76,10 +79,14 @@ int main(int argc, char **argv, char **envp)
  textdomain(PACKAGE);
 #endif
 
+#ifdef REQUIRE_GLIB_TYPE_INIT
+ g_type_init();
+#endif
+
  cfgdirname = g_strdup_printf("%s/%s", g_get_user_config_dir(), PACKAGE);
  if(!cfgdirname) g_error(_("Out of memory."));
 
- cfgfilename = g_strdup_printf("%s/%s", cfgdirname, CFGFILENAME);
+ cfgfilename = g_strdup_printf("%s/apt-dater.xml", cfgdirname);
  if(!cfgfilename) g_error(_("Out of memory."));
 
  g_set_prgname(PACKAGE);
@@ -121,8 +128,10 @@ int main(int argc, char **argv, char **envp)
  }
  if(!cfgfilename) g_error(_("Out of memory."));
 
- if(!(cfg = (CfgFile *) loadConfig(cfgfilename))) {
-  g_error(_("Error on loading config file %s\n"), cfgfilename);
+ cfg = initialConfig();
+
+ if(!(loadConfig(cfgfilename, cfg))) {
+  g_printerr(_("Error on loading config file %s\n"), cfgfilename);
   exit(EXIT_FAILURE);
  }
 
@@ -152,23 +161,25 @@ int main(int argc, char **argv, char **envp)
   }
   /* Add keys */
   else {
-    gint i;
-    gchar **add_argv = g_new0(gchar *, cfg->ssh_numadd+2);
+    if(cfg->ssh_numadd) {
+	gint i;
+	gchar **add_argv = g_new0(gchar *, cfg->ssh_numadd+2);
 
-    add_argv[0] = "ssh-add";
+	add_argv[0] = PKGLIBDIR"/ssh-addonce";
 
-    for(i=0; i<cfg->ssh_numadd; i++)
-     add_argv[i+1] = cfg->ssh_add[i];
+	for(i=0; i<cfg->ssh_numadd; i++)
+	 add_argv[i+1] = cfg->ssh_add[i];
 
-    GError *error;
-    if(g_spawn_sync(NULL, add_argv, NULL, 
-         G_SPAWN_SEARCH_PATH | G_SPAWN_CHILD_INHERITS_STDIN, NULL, NULL,
-         NULL, NULL, NULL, &error) == FALSE) {
-      g_warning("%s", error->message);
-      g_clear_error (&error);
+	GError *error = NULL;
+	if(g_spawn_sync(NULL, add_argv, NULL,
+	     G_SPAWN_SEARCH_PATH | G_SPAWN_CHILD_INHERITS_STDIN, NULL, NULL,
+	     NULL, NULL, NULL, &error) == FALSE) {
+	  g_warning("%s", error->message);
+	  g_clear_error (&error);
+	}
+
+	g_free(add_argv);
     }
-
-    g_free(add_argv);
   }
  }
 
@@ -177,12 +188,15 @@ int main(int argc, char **argv, char **envp)
 #ifdef FEAT_XMLREPORT
  if(!report) {
 #endif
+
+#ifndef FEAT_TMUX
 #ifdef __linux__
    /* Test if we are the owner of the TTY or die. */
    if(g_access("/proc/self/fd/0", R_OK|W_OK)) {
      g_error(_("Cannot open your terminal /proc/self/fd/0 - please check."));
      exit(EXIT_FAILURE);
    }
+#endif
 #endif
 
    getOldestMtime(hosts);
@@ -201,12 +215,13 @@ int main(int argc, char **argv, char **envp)
 
  loop = g_main_loop_new (NULL, FALSE);
 
-#ifdef HAVE_GLIB_TIMEOUT_ADD_SECONDS
- g_timeout_add_seconds(1, (GSourceFunc) refreshStats, hosts);
+ /* tmux supports GFileMonitor, screen requires polling */
+#ifdef FEAT_TMUX
+ refreshStats(hosts);
 #else
- g_timeout_add(1000, (GSourceFunc) refreshStats, hosts);
+ g_timeout_add_seconds(1, (GSourceFunc) refreshStats, hosts);
 #endif
- 
+
 #ifdef FEAT_XMLREPORT
  if(report)
   g_idle_add ((GSourceFunc) ctrlReport, hosts);
