@@ -1,11 +1,10 @@
 /* apt-dater - terminal-based remote package update manager
  *
  * Authors:
- *   Andre Ellguth <ellguth@ibh.de>
  *   Thomas Liske <liske@ibh.de>
  *
  * Copyright Holder:
- *   2008-2012 (C) IBH IT-Service GmbH [http://www.ibh.de/apt-dater/]
+ *   2008-2015 (C) IBH IT-Service GmbH [https://www.ibh.de/apt-dater/]
  *
  * License:
  *   This program is free software; you can redistribute it and/or modify
@@ -33,7 +32,7 @@
 #include "apt-dater.h"
 #include "ui.h"
 #include "colors.h"
-#include "screen.h"
+#include "ttymux.h"
 #include "exec.h"
 #include "stats.h"
 #include "keyfiles.h"
@@ -89,6 +88,7 @@ typedef enum {
     TCLMK_CATEGORY,
     TCLMK_GROUP,
     TCLMK_HOSTNAME,
+    TCLMK_COMMENT,
     TCLMK_KERNEL,
     TCLMK_LSBCNAME,
     TCLMK_LSBDISTRI,
@@ -119,6 +119,7 @@ const static struct TCLMapping tclmap[] = {
     {TCLMK_CATEGORY , "cat"       , TCLM_INT},
     {TCLMK_GROUP    , "group"     , TCLM_STRING},
     {TCLMK_HOSTNAME , "hostname"  , TCLM_STRING},
+    {TCLMK_COMMENT  , "comment"   , TCLM_STRING},
     {TCLMK_KERNEL   , "kernel"    , TCLM_STRING},
     {TCLMK_LSBCNAME , "lsb_cname" , TCLM_STRING},
     {TCLMK_LSBDISTRI, "lsb_distri", TCLM_STRING},
@@ -205,8 +206,9 @@ static struct ShortCut shortCuts[] = {
 const struct HostFlag hostFlags[] = {
   {HOST_STATUS_PKGKEPTBACK    ,  "h", N_("some packages are kept back")},
   {HOST_STATUS_PKGEXTRA       ,  "x", N_("extra packages are installed")},
-  {HOST_STATUS_KERNELNOTMATCH ,  "R", N_("running kernel is not the latest (reboot required)")},
-  {HOST_STATUS_KERNELSELFBUILD,  "k", N_("a selfbuild kernel is running")},
+  {HOST_STATUS_KERNELABIUPGR  ,  "k", N_("pending kernel upgrade (ABI compatible)")},
+  {HOST_STATUS_KERNELVERUPGR  ,  "K", N_("pending kernel upgrade")},
+  {HOST_STATUS_KERNELUNKNOWN  ,  "?", N_("unknown kernel upgrade state")},
   {HOST_STATUS_VIRTUALIZED    ,  "v", N_("this is a virtualized machine")},
 #ifdef FEAT_CLUSTERS
   {HOST_STATUS_CLUSTERED      ,  "C", N_("this machine is part of a cluster")},
@@ -603,7 +605,7 @@ static void drawHelp ()
 
  i = -1;
  while(shortCuts[++i].key) {
-  mvwaddnstr(wp, l,  2, _(shortCuts[i].key)  , COLS - 2);
+  mvwaddnstr(wp, l,  2, shortCuts[i].key  , COLS - 2);
   mvwaddnstr(wp, l, 16, _(shortCuts[i].descr), COLS - 16);
        
   l++;
@@ -653,6 +655,10 @@ static void drawHostDetails(HostNode *h)
  mvwaddnstr(wp, l++, 20, h->group, COLS - 20);
  mvwaddnstr(wp, l  ,  2, _("Hostname:"), COLS -  2);
  mvwaddnstr(wp, l++, 20, h->hostname, COLS - 20);
+ if (h->comment) {
+   mvwaddnstr(wp, l  ,  2, _("Comment:"), COLS -  2);
+   mvwaddnstr(wp, l++, 20, h->comment, COLS - 20);
+ }
  if (h->virt) {
   mvwaddnstr(wp, l  ,  2, _("Machine Type:"), COLS -  2);
   mvwaddnstr(wp, l++, 20, h->virt, COLS - 20);
@@ -709,12 +715,12 @@ static void drawHostDetails(HostNode *h)
   mvwaddnstr(wp, l  ,  2, _("Kernel version:"), COLS -  2);
   mvwaddnstr(wp, l++, 20, h->kernelrel, COLS - 20);
 	
-  switch(h->status & (HOST_STATUS_KERNELNOTMATCH | HOST_STATUS_KERNELSELFBUILD)) {
-  case HOST_STATUS_KERNELNOTMATCH:
-   strcpy(buf, _("(reboot required)"));
+  switch(h->status & (HOST_STATUS_KERNELABIUPGR | HOST_STATUS_KERNELVERUPGR)) {
+  case HOST_STATUS_KERNELABIUPGR:
+   strcpy(buf, _("(pending ABI compatible upgrade)"));
    break;
-  case HOST_STATUS_KERNELSELFBUILD:
-   strcpy(buf, _("(selfbuild kernel)"));
+  case HOST_STATUS_KERNELVERUPGR:
+   strcpy(buf, _("(pending upgrade)"));
    break;
   default:
    buf[0] = 0;
@@ -934,13 +940,13 @@ static void drawHostErrDiag(HostNode *h)
 
 void drawStatus (char *str, gboolean drawinfo)
 {
- char strinfo[30];
+ char strinfo[42];
  struct tm *tm_mtime;
  int mtime_pos;
 
  if(drawinfo == TRUE) {
     tm_mtime = localtime(&oldest_st_mtime);
-    strftime(strinfo, sizeof(strinfo), _(" [Oldest: %D %H:%M]"), tm_mtime);
+    strftime(strinfo, sizeof(strinfo), _(" [Oldest: %x %X]"), tm_mtime);
  }
 
  attron(uicolors[UI_COLOR_STATUS]);
@@ -1102,7 +1108,7 @@ static void drawHistoryEntries (HostNode *n)
 
  if(!n) return;
 
- hel = history_get_entries(inhost);
+ hel = history_get_entries(cfg, inhost);
  if(!hel) {
   drawQuery(_("No history data available!"), G_USEC_PER_SEC);
   refresh();
@@ -1478,7 +1484,7 @@ void drawPackageEntry (DrawNode *n)
 
 static gboolean refreshDumpWindow (DrawNode *n)
 {
- gchar *dump = n->type == SESSION ? screen_get_dump((SessNode *) n->p) : NULL;
+ gchar *dump = n->type == SESSION ? TTYMUX_GET_DUMP((SessNode *) n->p) : NULL;
  gchar *hostname = NULL;
  char   h[BUF_MAX_LEN];
  
@@ -1519,7 +1525,7 @@ void drawSessionEntry (DrawNode *n)
  strftime(&h[strlen(h)], sizeof(h)-strlen(h), _("%D %H:%M "), tm_mtime);
 
  snprintf(&h[strlen(h)], sizeof(h)-strlen(h), "(%s)",
-	  (screen_is_attached((SessNode *) n->p) ? _("Attached") : _("Detached")));
+	  (TTYMUX_IS_ATTACHED((SessNode *) n->p) ? _("Attached") : _("Detached")));
 
 
  attron(n->attrs);
@@ -1534,7 +1540,7 @@ void drawSessionEntry (DrawNode *n)
     drawQuery(_("Could not read session dump."), G_USEC_PER_SEC);
   } else {
    snprintf(statusln, sizeof(statusln), "Session is %s",
-	    (screen_is_attached((SessNode *) n->p) ? _("attached") : _("detached")));
+	    (TTYMUX_IS_ATTACHED((SessNode *) n->p) ? _("attached") : _("detached")));
    drawStatus(statusln, TRUE);
   }
  }
@@ -1592,7 +1598,7 @@ gchar *getStrFromDrawNode (DrawNode *n)
   strftime(&h[strlen(h)], sizeof(h)-strlen(h), _("%D %H:%M "), tm_mtime);
 
   snprintf(&h[strlen(h)], sizeof(h)-strlen(h), "(%s)",
-	   (screen_is_attached((SessNode *) n->p) ? _("Attached") : _("Detached")));  
+	   (TTYMUX_IS_ATTACHED((SessNode *) n->p) ? _("Attached") : _("Detached")));
 
   ret = h;
   break;
@@ -2717,6 +2723,9 @@ void applyFilter(GList *hosts) {
 	    case TCLMK_HOSTNAME:
         	Tcl_SetVar(tcl_interp, tclmap[i].name, n->hostname, 0);
 		break;
+	    case TCLMK_COMMENT:
+	        Tcl_SetVar(tcl_interp, tclmap[i].name, n->comment, 0);
+		break;
 	    case TCLMK_KERNEL:
         	Tcl_SetVar(tcl_interp, tclmap[i].name, n->kernelrel, 0);
 		break;
@@ -2914,7 +2923,7 @@ static void filterHosts(GList *hosts)
 #ifdef FEAT_HISTORY
 static void handleErrors(HostNode *n) {
  int c;
- HistoryEntry *he = history_recent_entry(n);
+ HistoryEntry *he = history_recent_entry(cfg, n);
 
  if(!he)
   return;
@@ -3084,6 +3093,9 @@ gboolean ctrlUI (GList *hosts)
        refscr = TRUE;
       }
      }
+#ifdef FEAT_TMUX
+   refreshStats(hosts);
+#endif
    break; /* case SC_KEY_REFRESH */
 
   case SC_KEY_REFRESH+TAGGED_MASK:
@@ -3122,6 +3134,9 @@ gboolean ctrlUI (GList *hosts)
      g_list_free (thosts);
     }
    }
+#ifdef FEAT_TMUX
+   refreshStats(hosts);
+#endif
    break; /* case SC_KEY_REFRESH + KEY_TAGGED_MASK */
 
   case SC_KEY_TAGACTION:
@@ -3194,6 +3209,9 @@ gboolean ctrlUI (GList *hosts)
     rebuildDrawList(hosts);
     initUI();
     refscr = TRUE;
+#ifdef FEAT_TMUX
+    refreshStats(hosts);
+#endif
     break;
    case CATEGORY:
    case GROUP:
@@ -3502,12 +3520,12 @@ gboolean ctrlUI (GList *hosts)
        continue;
       }
 
-      if (!screen_is_attached((SessNode *)scr->data)) {
+      if (!TTYMUX_IS_ATTACHED((SessNode *)scr->data)) {
        cleanUI();
 #ifdef FEAT_HISTORY
        gboolean f =
 #endif
-       screen_attach(m, (SessNode *)scr->data, FALSE);
+       TTYMUX_ATTACH(m, (SessNode *)scr->data, FALSE);
 #ifdef FEAT_HISTORY
        if(f)
         handleErrors(m);
@@ -3528,6 +3546,9 @@ gboolean ctrlUI (GList *hosts)
       ho = g_list_next(ho);
     }
    }
+#ifdef FEAT_TMUX
+   refreshStats(hosts);
+#endif
    break; /* case SC_KEY_NEXTSESS */
 
   case SC_KEY_ATTACH:
@@ -3552,7 +3573,7 @@ gboolean ctrlUI (GList *hosts)
     gboolean may_share = FALSE;
 
     /* Session already attached! */
-    if (screen_is_attached(s)) {
+    if (TTYMUX_IS_ATTACHED(s)) {
      if (!queryConfirm(_("Already attached - share session? [y/N]: "), FALSE, NULL))
       break;
 
@@ -3564,7 +3585,7 @@ gboolean ctrlUI (GList *hosts)
 #ifdef FEAT_HISTORY
     gboolean f =
 #endif
-    screen_attach(inhost, s, may_share);
+    TTYMUX_ATTACH(inhost, s, may_share);
 #ifdef FEAT_HISTORY
     if(f)
      handleErrors(inhost);
@@ -3572,6 +3593,9 @@ gboolean ctrlUI (GList *hosts)
     initUI();
    }
    refscr = TRUE;
+#ifdef FEAT_TMUX
+   refreshStats(hosts);
+#endif
    break; /* case SC_KEY_ATTACH */
 
   case SC_KEY_TOGGLEDUMPS:
